@@ -4,11 +4,13 @@
  * author: Ryan Dickie (c) 2008
  */
 
+
 #include "allegro5/allegro.h"
 #include "allegro5/allegro_acodec.h"
 #include "allegro5/allegro_audio.h"
 #include "allegro5/internal/aintern.h"
 #include "allegro5/internal/aintern_audio.h"
+#include "allegro5/internal/aintern_exitfunc.h"
 #include "allegro5/internal/aintern_system.h"
 #include "acodec.h"
 #include "helper.h"
@@ -416,11 +418,8 @@ static size_t ogg_stream_update(ALLEGRO_AUDIO_STREAM *stream, void *data,
 #endif
       pos += read;
 	   
-      /* If nothing read then now to silence from here to the end. */
       if (read == 0) {
-         int silence = _al_kcm_get_silence(stream->spl.spl_data.depth);
-         memset((char *)data + pos, silence, buf_size - pos);
-         /* return the number of usefull byes written */
+         /* Return the number of useful bytes written. */
          return pos;
       }
    }
@@ -514,7 +513,6 @@ ALLEGRO_AUDIO_STREAM *_al_load_ogg_vorbis_audio_stream_f(ALLEGRO_FILE *file,
 
    extra->loop_start = 0.0;
    extra->loop_end = ogg_stream_get_length(stream);
-   stream->feed_thread = al_create_thread(_al_kcm_feed_stream, stream);
    stream->quit_feed_thread = false;
    stream->feeder = ogg_stream_update;
    stream->rewind_feeder = ogg_stream_rewind;
@@ -523,7 +521,7 @@ ALLEGRO_AUDIO_STREAM *_al_load_ogg_vorbis_audio_stream_f(ALLEGRO_FILE *file,
    stream->get_feeder_length = ogg_stream_get_length;
    stream->set_feeder_loop = ogg_stream_set_loop;
    stream->unload_feeder = ogg_stream_close;
-   al_start_thread(stream->feed_thread);
+   _al_acodec_start_feed_thread(stream);
 	
    return stream;
 }
