@@ -26,6 +26,7 @@
 
 #include "allegro5/allegro_font.h"
 #include "allegro5/internal/aintern_dtor.h"
+#include "allegro5/internal/aintern_font.h"
 #include "allegro5/internal/aintern_system.h"
 
 /* If you call this, you're probably making a mistake. */
@@ -359,7 +360,7 @@ void al_destroy_font(ALLEGRO_FONT *f)
    if (!f)
       return;
 
-   _al_unregister_destructor(_al_dtor_list, f);
+   _al_unregister_destructor(_al_dtor_list, f->dtor_item);
 
    f->vtable->destroy(f);
 }
@@ -402,6 +403,12 @@ int al_get_glyph_advance(const ALLEGRO_FONT *f, int codepoint1, int codepoint2)
    return f->vtable->get_glyph_advance(f, codepoint1, codepoint2);
 }
 
+/* Function: al_get_glyph
+ */
+bool al_get_glyph(const ALLEGRO_FONT *f, int prev_codepoint, int codepoint, ALLEGRO_GLYPH *glyph)
+{
+   return f->vtable->get_glyph(f, prev_codepoint, codepoint, glyph);
+};
 
 
 /* This helper function helps splitting an ustr in several delimited parts.
