@@ -649,7 +649,7 @@ void al_destroy_menu(ALLEGRO_MENU *menu)
       ASSERT(false);
       return;
    }
-   else if (menu->display) {
+   else if (menu->display && !menu->is_popup_menu) {
       /* This is an active, top-level menu. */
       al_remove_display_menu(menu->display);
    }
@@ -723,6 +723,12 @@ bool al_set_display_menu(ALLEGRO_DISPLAY *display, ALLEGRO_MENU *menu)
 {
    DISPLAY_MENU *dm = NULL;
    size_t i;
+   int menu_height = _al_get_menu_display_height();
+   bool automatic_menu_display_resize = true;
+   const char* automatic_menu_display_resize_value =
+      al_get_config_value(al_get_system_config(), "compatibility", "automatic_menu_display_resize");
+   if (automatic_menu_display_resize_value && strcmp(automatic_menu_display_resize_value, "false") == 0)
+      automatic_menu_display_resize = false; 
 
    ASSERT(display);
 
@@ -741,11 +747,16 @@ bool al_set_display_menu(ALLEGRO_DISPLAY *display, ALLEGRO_MENU *menu)
       /* Removing the menu */
 
       if (!dm)
-         return false; 
+         return false;
 
       _al_hide_display_menu(display, dm->menu);
       _al_walk_over_menu(dm->menu, set_menu_display_r, NULL);
       _al_vector_delete_at(&display_menus, i);
+
+      if (automatic_menu_display_resize && menu_height > 0) {
+         display->extra_resize_height = 0;
+         al_resize_display(display, al_get_display_width(display), al_get_display_height(display));
+      }
    }
    else {
       /* Setting the menu. It must not currently be attached to any 
@@ -773,6 +784,15 @@ bool al_set_display_menu(ALLEGRO_DISPLAY *display, ALLEGRO_MENU *menu)
       if (!dm)
          dm = _al_vector_alloc_back(&display_menus);
 
+      if (automatic_menu_display_resize && menu_height > 0) {
+         /* Temporarily disable the constraints so we don't send a RESIZE_EVENT. */
+         bool old_constraints = display->use_constraints;
+         display->use_constraints = false;
+         display->extra_resize_height = menu_height;
+         al_resize_display(display, al_get_display_width(display), al_get_display_height(display));
+         display->use_constraints = old_constraints;
+      }
+
       dm->display = display;
       dm->menu = menu;
    }
@@ -784,6 +804,7 @@ bool al_set_display_menu(ALLEGRO_DISPLAY *display, ALLEGRO_MENU *menu)
  */
 bool al_popup_menu(ALLEGRO_MENU *popup, ALLEGRO_DISPLAY *display)
 {
+   bool ret;
    ASSERT(popup);
 
    if (!popup->is_popup_menu || popup->parent)
@@ -795,7 +816,12 @@ bool al_popup_menu(ALLEGRO_MENU *popup, ALLEGRO_DISPLAY *display)
    /* Set the entire menu tree as owned by the display */
    _al_walk_over_menu(popup, set_menu_display_r, display);
 
-   return _al_show_popup_menu(display, popup);
+   ret = _al_show_popup_menu(display, popup);
+
+   if (!ret) {
+      _al_walk_over_menu(popup, set_menu_display_r, NULL);
+   }
+   return ret;
 }
 
 /* Function: al_remove_display_menu
