@@ -463,6 +463,11 @@ bool _al_show_popup_menu(ALLEGRO_DISPLAY *display, ALLEGRO_MENU *amenu)
     return true;
 }
 
+int _al_get_menu_display_height(void)
+{
+   return 0;
+}
+
 @implementation ALLEGMenuTarget
 /* Initial conversion of ALLEGRO_MENU_ITEM to NSMenuItem.
  * The target (self) is set for the item.
