@@ -6,6 +6,7 @@
  *    source to communicate back to the main program.
  */
 
+#define ALLEGRO_UNSTABLE
 #include <stdio.h>
 #include <allegro5/allegro.h>
 #include <allegro5/allegro_native_dialog.h>
@@ -152,7 +153,7 @@ static void show_files_list(ALLEGRO_FILECHOOSER *dialog,
 }
 
 
-int main(void)
+int main(int argc, char **argv)
 {
    ALLEGRO_DISPLAY *display;
    ALLEGRO_TIMER *timer;
@@ -166,6 +167,10 @@ int main(void)
    bool close_log = false;
    int button;
    bool message_log = true;
+   bool touch;
+
+   (void)argc;
+   (void)argv;
 
    if (!al_init()) {
       abort_example("Could not init Allegro.\n");
@@ -186,9 +191,18 @@ int main(void)
    info = al_color_name("red");
 
    al_install_mouse();
+   touch = al_install_touch_input();
    al_install_keyboard();
+    
+   if (touch) {
+      al_set_mouse_emulation_mode(ALLEGRO_MOUSE_EMULATION_5_0_x);
+   }
 
-   message("Creating 640x480 window...");
+   message("Creating window...");
+    
+#ifdef ALLEGRO_IPHONE
+   al_set_new_display_flags(ALLEGRO_FULLSCREEN_WINDOW);
+#endif
 
    display = al_create_display(640, 480);
    if (!display) {
@@ -211,6 +225,10 @@ restart:
    queue = al_create_event_queue();
    al_register_event_source(queue, al_get_keyboard_event_source());
    al_register_event_source(queue, al_get_mouse_event_source());
+   if (touch) {
+      al_register_event_source(queue, al_get_touch_input_event_source());
+      al_register_event_source(queue, al_get_touch_input_mouse_emulation_event_source());
+   }
    al_register_event_source(queue, al_get_display_event_source(display));
    al_register_event_source(queue, al_get_timer_event_source(timer));
    if (textlog) {
@@ -231,7 +249,7 @@ restart:
          if (event.keyboard.keycode == ALLEGRO_KEY_ESCAPE && !cur_dialog)
             break;
       }
-
+       
       /* When a mouse button is pressed, and no native dialog is
        * shown already, we show a new one.
        */
