@@ -1,6 +1,6 @@
-/*         ______   ___    ___ 
+/*         ______   ___    ___
  *        /\  _  \ /\_ \  /\_ \
- *        \ \ \L\ \\//\ \ \//\ \      __     __   _ __   ___ 
+ *        \ \ \L\ \\//\ \ \//\ \      __     __   _ __   ___
  *         \ \  __ \ \ \ \  \ \ \   /'__`\ /'_ `\/\`'__\/ __`\
  *          \ \ \/\ \ \_\ \_ \_\ \_/\  __//\ \L\ \ \ \//\ \L\ \
  *           \ \_\ \_\/\____\/\____\ \____\ \____ \ \_\\ \____/
@@ -31,7 +31,7 @@ void al_copy_transform(ALLEGRO_TRANSFORM *dest, const ALLEGRO_TRANSFORM *src)
 {
    ASSERT(src);
    ASSERT(dest);
-   
+
    memcpy(dest, src, sizeof(ALLEGRO_TRANSFORM));
 }
 
@@ -53,7 +53,7 @@ void al_use_transform(const ALLEGRO_TRANSFORM *trans)
 
    if (trans != &target->transform) {
       al_copy_transform(&target->transform, trans);
-      
+
       target->inverse_transform_dirty = true;
    }
 
@@ -131,7 +131,7 @@ const ALLEGRO_TRANSFORM *al_get_current_inverse_transform(void)
 
    if (!target)
       return NULL;
-      
+
    if (target->inverse_transform_dirty) {
       al_copy_transform(&target->inverse_transform, &target->transform);
       al_invert_transform(&target->inverse_transform);
@@ -145,22 +145,22 @@ const ALLEGRO_TRANSFORM *al_get_current_inverse_transform(void)
 void al_identity_transform(ALLEGRO_TRANSFORM *trans)
 {
    ASSERT(trans);
-   
+
    trans->m[0][0] = 1;
    trans->m[0][1] = 0;
    trans->m[0][2] = 0;
    trans->m[0][3] = 0;
-   
+
    trans->m[1][0] = 0;
    trans->m[1][1] = 1;
    trans->m[1][2] = 0;
    trans->m[1][3] = 0;
-   
+
    trans->m[2][0] = 0;
    trans->m[2][1] = 0;
    trans->m[2][2] = 1;
    trans->m[2][3] = 0;
-   
+
    trans->m[3][0] = 0;
    trans->m[3][1] = 0;
    trans->m[3][2] = 0;
@@ -174,25 +174,25 @@ void al_build_transform(ALLEGRO_TRANSFORM *trans, float x, float y,
 {
    float c, s;
    ASSERT(trans);
-   
+
    c = cosf(theta);
    s = sinf(theta);
-   
+
    trans->m[0][0] = sx * c;
    trans->m[0][1] = sy * s;
    trans->m[0][2] = 0;
    trans->m[0][3] = 0;
-   
+
    trans->m[1][0] = -sx * s;
    trans->m[1][1] = sy * c;
    trans->m[1][2] = 0;
    trans->m[1][3] = 0;
-   
+
    trans->m[2][0] = 0;
    trans->m[2][1] = 0;
    trans->m[2][2] = 1;
    trans->m[2][3] = 0;
-   
+
    trans->m[3][0] = x;
    trans->m[3][1] = y;
    trans->m[3][2] = 0;
@@ -273,7 +273,7 @@ void al_invert_transform(ALLEGRO_TRANSFORM *trans)
 {
    float det, t;
    ASSERT(trans);
-   
+
    det =  trans->m[0][0] *  trans->m[1][1] -  trans->m[1][0] *  trans->m[0][1];
 
    t =  trans->m[3][0];
@@ -283,7 +283,7 @@ void al_invert_transform(ALLEGRO_TRANSFORM *trans)
    t =  trans->m[0][0];
    trans->m[0][0] =  trans->m[1][1] / det;
    trans->m[1][1] = t / det;
-   
+
    trans->m[0][1] = - trans->m[0][1] / det;
    trans->m[1][0] = - trans->m[1][0] / det;
 }
@@ -294,7 +294,7 @@ int al_check_inverse(const ALLEGRO_TRANSFORM *trans, float tol)
 {
    float det, norm, c0, c1, c3;
    ASSERT(trans);
-   
+
    det = fabsf(trans->m[0][0] *  trans->m[1][1] -  trans->m[1][0] *  trans->m[0][1]);
    /*
    We'll use the 1-norm, as it is the easiest to compute
@@ -312,7 +312,7 @@ int al_check_inverse(const ALLEGRO_TRANSFORM *trans, float tol)
 void al_translate_transform(ALLEGRO_TRANSFORM *trans, float x, float y)
 {
    ASSERT(trans);
-   
+
    trans->m[3][0] += x;
    trans->m[3][1] += y;
 }
@@ -324,7 +324,7 @@ void al_translate_transform_3d(ALLEGRO_TRANSFORM *trans, float x, float y,
     float z)
 {
    ASSERT(trans);
-   
+
    trans->m[3][0] += x;
    trans->m[3][1] += y;
    trans->m[3][2] += z;
@@ -338,18 +338,18 @@ void al_rotate_transform(ALLEGRO_TRANSFORM *trans, float theta)
    float c, s;
    float t;
    ASSERT(trans);
-   
+
    c = cosf(theta);
    s = sinf(theta);
-   
+
    t = trans->m[0][0];
    trans->m[0][0] = t * c - trans->m[0][1] * s;
    trans->m[0][1] = t * s + trans->m[0][1] * c;
-   
+
    t = trans->m[1][0];
    trans->m[1][0] = t * c - trans->m[1][1] * s;
    trans->m[1][1] = t * s + trans->m[1][1] * c;
-   
+
    t = trans->m[3][0];
    trans->m[3][0] = t * c - trans->m[3][1] * s;
    trans->m[3][1] = t * s + trans->m[3][1] * c;
@@ -360,13 +360,13 @@ void al_rotate_transform(ALLEGRO_TRANSFORM *trans, float theta)
 void al_scale_transform(ALLEGRO_TRANSFORM *trans, float sx, float sy)
 {
    ASSERT(trans);
-   
+
    trans->m[0][0] *= sx;
    trans->m[0][1] *= sy;
-   
+
    trans->m[1][0] *= sx;
    trans->m[1][1] *= sy;
-   
+
    trans->m[3][0] *= sx;
    trans->m[3][1] *= sy;
 }
@@ -378,11 +378,11 @@ void al_scale_transform_3d(ALLEGRO_TRANSFORM *trans, float sx, float sy,
     float sz)
 {
    ASSERT(trans);
-   
+
    trans->m[0][0] *= sx;
    trans->m[0][1] *= sy;
    trans->m[0][2] *= sz;
-   
+
    trans->m[1][0] *= sx;
    trans->m[1][1] *= sy;
    trans->m[1][2] *= sz;
@@ -390,7 +390,7 @@ void al_scale_transform_3d(ALLEGRO_TRANSFORM *trans, float sx, float sy,
    trans->m[2][0] *= sx;
    trans->m[2][1] *= sy;
    trans->m[2][2] *= sz;
-   
+
    trans->m[3][0] *= sx;
    trans->m[3][1] *= sy;
    trans->m[3][2] *= sz;
@@ -406,7 +406,7 @@ void al_transform_coordinates(const ALLEGRO_TRANSFORM *trans, float *x, float *y
    ASSERT(y);
 
    t = *x;
-   
+
    *x = t * trans->m[0][0] + *y * trans->m[1][0] + trans->m[3][0];
    *y = t * trans->m[0][1] + *y * trans->m[1][1] + trans->m[3][1];
 }
@@ -435,6 +435,45 @@ void al_transform_coordinates_3d(const ALLEGRO_TRANSFORM *trans,
    *z = rz;
 }
 
+/* Function: al_transform_coordinates_4d
+ */
+void al_transform_coordinates_4d(const ALLEGRO_TRANSFORM *trans,
+   float *x, float *y, float *z, float *w)
+{
+   float rx, ry, rz, rw;
+   ASSERT(trans);
+   ASSERT(x);
+   ASSERT(y);
+   ASSERT(z);
+   ASSERT(w);
+
+   #define M(i, j) trans->m[i][j]
+
+   rx = M(0, 0) * *x + M(1, 0) * *y + M(2, 0) * *z + M(3, 0) * *w;
+   ry = M(0, 1) * *x + M(1, 1) * *y + M(2, 1) * *z + M(3, 1) * *w;
+   rz = M(0, 2) * *x + M(1, 2) * *y + M(2, 2) * *z + M(3, 2) * *w;
+   rw = M(0, 3) * *x + M(1, 3) * *y + M(2, 3) * *z + M(3, 3) * *w;
+
+   #undef M
+
+   *x = rx;
+   *y = ry;
+   *z = rz;
+   *w = rw;
+}
+
+/* Function: al_transform_coordinates_3d_projective
+ */
+void al_transform_coordinates_3d_projective(const ALLEGRO_TRANSFORM *trans,
+   float *x, float *y, float *z)
+{
+   float w = 1;
+   al_transform_coordinates_4d(trans, x, y, z, &w);
+   *x /= w;
+   *y /= w;
+   *z /= w;
+}
+
 /* Function: al_compose_transform
  */
 void al_compose_transform(ALLEGRO_TRANSFORM *trans, const ALLEGRO_TRANSFORM *other)
