@@ -3,25 +3,33 @@
 #include "allegro5/allegro.h"
 #include "allegro5/platform/aintunix.h"
 #include "allegro5/internal/aintern_x.h"
+#include "allegro5/internal/aintern_xclipboard.h"
 #include "allegro5/internal/aintern_xdisplay.h"
+#include "allegro5/internal/aintern_xembed.h"
 #include "allegro5/internal/aintern_xevents.h"
 #include "allegro5/internal/aintern_xfullscreen.h"
 #include "allegro5/internal/aintern_xkeyboard.h"
 #include "allegro5/internal/aintern_xmouse.h"
 #include "allegro5/internal/aintern_xsystem.h"
+#include "allegro5/internal/aintern_xtouch.h"
+
+#ifdef ALLEGRO_RASPBERRYPI
+#include "allegro5/internal/aintern_raspberrypi.h"
+#define ALLEGRO_SYSTEM_XGLX ALLEGRO_SYSTEM_RASPBERRYPI
+#define ALLEGRO_DISPLAY_XGLX ALLEGRO_DISPLAY_RASPBERRYPI
+#endif
 
 ALLEGRO_DEBUG_CHANNEL("xevents")
 
 /* Handle an X11 close button event. [X11 thread]
  * Only called from the event handler with the system locked.
  */
-static void _al_display_xglx_closebutton(ALLEGRO_DISPLAY *d, XEvent *xevent)
+void _al_display_xglx_closebutton(ALLEGRO_DISPLAY *d, XEvent *xevent)
 {
    ALLEGRO_EVENT_SOURCE *es = &d->es;
    (void)xevent;
 
    _al_event_source_lock(es);
-
    if (_al_event_source_needs_to_generate_event(es)) {
       ALLEGRO_EVENT event;
       event.display.type = ALLEGRO_EVENT_DISPLAY_CLOSE;
@@ -76,10 +84,42 @@ static void process_x11_event(ALLEGRO_SYSTEM_XGLX *s, XEvent event)
             d->mouse_warp = true;
             break;
          }
-         if ((Atom)event.xclient.data.l[0] == d->wm_delete_window_atom) {
+         if (d->wm_delete_window_atom != None &&
+            (Atom)event.xclient.data.l[0] == d->wm_delete_window_atom)
+         {
             _al_display_xglx_closebutton(&d->display, &event);
             break;
          }
+#ifndef ALLEGRO_RASPBERRYPI
+         if (event.xclient.message_type == s->XEmbedAtom) {
+            const long xtime = event.xclient.data.l[0];
+            const long major = event.xclient.data.l[1];
+            const long detail = event.xclient.data.l[2];
+            const long data1 = event.xclient.data.l[3];
+            const long data2 = event.xclient.data.l[4];
+
+            (void)xtime;
+            (void)detail;
+            (void)data2;
+
+            switch (major) {
+               case XEMBED_EMBEDDED_NOTIFY:
+                  d->embedder_window = data1;
+                  ALLEGRO_INFO("XEmbed begin: embedder window = %ld\n", data1);
+                  break;
+               case XEMBED_FOCUS_IN:
+                  ALLEGRO_DEBUG("XEmbed focus in\n");
+                  _al_xwin_display_switch_handler_inner(&d->display, true);
+                  _al_xwin_keyboard_switch_handler(&d->display, true);
+                  break;
+               case XEMBED_FOCUS_OUT:
+                  ALLEGRO_DEBUG("XEmbed focus out\n");
+                  _al_xwin_display_switch_handler_inner(&d->display, false);
+                  _al_xwin_keyboard_switch_handler(&d->display, false);
+                  break;
+            }
+            break;
+         }
          break;
       case EnterNotify:
          _al_xwin_mouse_switch_handler(&d->display, &event.xcrossing);
@@ -113,9 +153,28 @@ static void process_x11_event(ALLEGRO_SYSTEM_XGLX *s, XEvent event)
             _al_xwin_display_expose(&d->display, &event.xexpose);
          }
          break;
+      case ReparentNotify:
+         if (event.xreparent.parent == RootWindow(s->x11display, d->xscreen)) {
+            ALLEGRO_INFO("XEmbed protocol finished.\n");
+            d->embedder_window = None;
+         }
+         break;
+         
+      case SelectionNotify:        
+        _al_xwin_display_selection_notify(&d->display, &event.xselection);
+        d->is_selectioned = true;
+        _al_cond_signal(&d->selectioned);
+        break;
+         
+      case SelectionRequest:
+        _al_xwin_display_selection_request(&d->display, &event.xselectionrequest);
+        break;
+
       default:
+         _al_x_handle_touch_event(s, d, &event);
          _al_xglx_handle_mmon_event(s, d, &event);
          break;
+#endif
    }
 }
 
