@@ -1,7 +1,16 @@
-#include "allegro5/internal/aintern_xglx.h"
+#include "allegro5/allegro.h"
+#include "allegro5/allegro_opengl.h"
+#include "allegro5/opengl/gl_ext.h"
+#include "allegro5/internal/aintern.h"
+#include "allegro5/internal/aintern_display.h"
+#include "allegro5/internal/aintern_opengl.h"
 #include "allegro5/internal/aintern_pixels.h"
+#include "allegro5/internal/aintern_x.h"
+#include "allegro5/internal/aintern_xdisplay.h"
+#include "allegro5/internal/aintern_xglx_config.h"
+#include "allegro5/internal/aintern_xsystem.h"
 
-ALLEGRO_DEBUG_CHANNEL("display")
+ALLEGRO_DEBUG_CHANNEL("xglx_config")
 
 #ifdef DEBUGMODE
 static void display_pixel_format(ALLEGRO_EXTRA_DISPLAY_SETTINGS *eds)
@@ -430,24 +439,21 @@ static void select_best_visual(ALLEGRO_DISPLAY_XGLX *glx,
 
 void _al_xglx_config_select_visual(ALLEGRO_DISPLAY_XGLX *glx)
 {
-   ALLEGRO_SYSTEM_XGLX *system = (void *)al_get_system_driver();
    ALLEGRO_EXTRA_DISPLAY_SETTINGS **eds;
    int eds_count, i;
    bool force_old = false;
    bool using_fbc;
 
-   if (system->system.config) {
-      const char *selection_mode;
-      selection_mode = al_get_config_value(system->system.config, "graphics",
-                          "config_selection");
-      if (selection_mode && selection_mode[0] != '\0') {
-         if (!_al_stricmp(selection_mode, "old")) {
-            ALLEGRO_WARN("Forcing OLD visual selection method.\n");
-            force_old = true;
-         }
-         else if (!_al_stricmp(selection_mode, "new"))
-            force_old = false;
+   const char *selection_mode;
+   selection_mode = al_get_config_value(al_get_system_config(), "graphics",
+                       "config_selection");
+   if (selection_mode && selection_mode[0] != '\0') {
+      if (!_al_stricmp(selection_mode, "old")) {
+         ALLEGRO_WARN("Forcing OLD visual selection method.\n");
+         force_old = true;
       }
+      else if (!_al_stricmp(selection_mode, "new"))
+         force_old = false;
    }
 
    if (glx->glx_version >= 130 && !force_old)
@@ -479,7 +485,7 @@ void _al_xglx_config_select_visual(ALLEGRO_DISPLAY_XGLX *glx)
 }
 
 static GLXContext create_context_new(int ver, Display *dpy, GLXFBConfig fb,
-   GLXContext ctx, bool forward_compat, int major, int minor)
+   GLXContext ctx, bool forward_compat, bool want_es, int major, int minor)
 {
    typedef GLXContext (*GCCA_PROC) (Display*, GLXFBConfig, GLXContext, Bool, const int*);
    GCCA_PROC _xglx_glXCreateContextAttribsARB = NULL;
@@ -501,9 +507,14 @@ static GLXContext create_context_new(int ver, Display *dpy, GLXFBConfig fb,
    int attrib[] = {GLX_CONTEXT_MAJOR_VERSION_ARB, major,
                    GLX_CONTEXT_MINOR_VERSION_ARB, minor,
                    GLX_CONTEXT_FLAGS_ARB, 0,
+                   0, 0,
                    0};
    if (forward_compat)
       attrib[5] = GLX_CONTEXT_FORWARD_COMPATIBLE_BIT_ARB;
+   if (want_es) {
+      attrib[6] = GLX_CONTEXT_PROFILE_MASK_ARB;
+      attrib[7] = GLX_CONTEXT_ES_PROFILE_BIT_EXT;
+   }
    return _xglx_glXCreateContextAttribsARB(dpy, fb, ctx, True, attrib);
 }
 
@@ -521,13 +532,34 @@ bool _al_xglx_config_create_context(ALLEGRO_DISPLAY_XGLX *glx)
          existing_ctx = (*existing_dpy)->context;
    }
 
+   int major = _al_get_suggested_display_option(disp,
+      ALLEGRO_OPENGL_MAJOR_VERSION, 0);
+   int minor = _al_get_suggested_display_option(disp,
+      ALLEGRO_OPENGL_MINOR_VERSION, 0);
+
    if (glx->fbc) {
+      bool forward_compat = (disp->flags & ALLEGRO_OPENGL_FORWARD_COMPATIBLE) != 0;
       /* Create a GLX context from FBC. */
-      if (disp->flags & ALLEGRO_OPENGL_3_0) {
-         bool forward_compat = (disp->flags & ALLEGRO_OPENGL_FORWARD_COMPATIBLE) != 0;
+      if (disp->flags & ALLEGRO_OPENGL_ES_PROFILE) {
+         if (major == 0)
+            major = 2;
          glx->context = create_context_new(glx->glx_version,
-            system->gfxdisplay, *glx->fbc, existing_ctx, forward_compat, 3, 0);
-         disp->extra_settings.settings[ALLEGRO_COMPATIBLE_DISPLAY] = !forward_compat;
+            system->gfxdisplay, *glx->fbc, existing_ctx, forward_compat,
+            true, major, minor);
+      }
+      else if ((disp->flags & ALLEGRO_OPENGL_3_0) || major != 0) {
+         if (major == 0)
+            major = 3;
+         glx->context = create_context_new(glx->glx_version,
+            system->gfxdisplay, *glx->fbc, existing_ctx, forward_compat,
+               false, major, minor);
+         /* TODO: Right now Allegro's own OpenGL driver only works with a 3.0+
+          * context when using the programmable pipeline, for some reason. All
+          * that's missing is probably a default shader though.
+          */
+         disp->extra_settings.settings[ALLEGRO_COMPATIBLE_DISPLAY] = 1;
+         if (forward_compat && !(disp->flags & ALLEGRO_PROGRAMMABLE_PIPELINE))
+            disp->extra_settings.settings[ALLEGRO_COMPATIBLE_DISPLAY] = 0;
       }
       else {
          glx->context = glXCreateNewContext(system->gfxdisplay, *glx->fbc,
